/*----------------------------------------------------------------------------
 * Name:    Blinky.c
 * Purpose: LED Flasher
 *----------------------------------------------------------------------------
 * This file is part of the uVision/ARM development tools.
 * This software may only be used under the terms of a valid, current,
 * end user licence from KEIL for a compatible version of KEIL software
 * development tools. Nothing else gives you the right to use this software.
 *
 * This software is supplied "AS IS" without warranties of any kind.
 *
 * Copyright (c) 2017 Keil - An ARM Company. All rights reserved.
 *----------------------------------------------------------------------------*/

/*
 * --------Included Headers--------
 */
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"

#include "RTE_Components.h"
#include CMSIS_device_header
#include "Board_LED.h"
#include "Board_Buttons.h"              /* ::Board Support:Buttons */
#include "Board_GLCD.h"                 /* ::Board Support:Graphic LCD */
#include "GLCD_Config.h"                /* Keil.V2M-MPS2::Board Support:Graphic LCD */

#define MS_TO_TICKS(MS)    ((MS)/portTICK_RATE_MS)

extern GLCD_FONT  GLCD_Font_16x24;
SemaphoreHandle_t xLCDMutex = NULL;
SemaphoreHandle_t xButtonLedSemaphore = NULL;

/*
 * --------Functions--------
 */

 /*----------------------------------------------------------------------------
  display Button pressed on GLCD
 *---------------------------------------------------------------------------*/
void prvButOnLcd( uint32_t ulButNum )
{
#ifdef __USE_LCD
	configASSERT( xSemaphoreTake( xLCDMutex, portMAX_DELAY ) == pdTRUE );
	GLCD_SetBackgroundColor( GLCD_COLOR_WHITE );
	GLCD_SetForegroundColor( GLCD_COLOR_BLACK );
	GLCD_DrawChar( ( 9 + ulButNum ) * 16, 7 * 24, 0x80 + 1 );     /* Circle full */
	xSemaphoreGive( xLCDMutex );
#endif
}

/*----------------------------------------------------------------------------
  display button released GLCD
 *---------------------------------------------------------------------------*/
void prvButOffLcd(uint32_t ulButNum)
{
#ifdef __USE_LCD
	configASSERT( xSemaphoreTake( xLCDMutex, portMAX_DELAY ) == pdTRUE );
	GLCD_SetBackgroundColor( GLCD_COLOR_WHITE );
	GLCD_SetForegroundColor( GLCD_COLOR_LIGHT_GREY );
	GLCD_DrawChar( ( 9 + ulButNum ) * 16, 7 * 24, 0x80 + 0 );     /* Circle full */
	xSemaphoreGive( xLCDMutex );
#endif
}

/*----------------------------------------------------------------------------
  display LED on GLCD
 *---------------------------------------------------------------------------*/
void prvLedOnLcd( uint32_t ulLedNum )
{
#ifdef __USE_LCD
	configASSERT( xSemaphoreTake( xLCDMutex, portMAX_DELAY ) == pdTRUE );
	GLCD_SetBackgroundColor( GLCD_COLOR_WHITE );
	GLCD_SetForegroundColor( GLCD_COLOR_GREEN );
	GLCD_DrawChar( ( 9 + ulLedNum ) * 16, 6 * 24, 0x80 + 1 );     /* Circle full */
	xSemaphoreGive( xLCDMutex );
#endif
}

/*----------------------------------------------------------------------------
  display LED off GLCD
 *---------------------------------------------------------------------------*/
void prvLedOffLcd ( uint32_t ulLedNum )
{
#ifdef __USE_LCD
	configASSERT( xSemaphoreTake( xLCDMutex, portMAX_DELAY ) == pdTRUE );
	GLCD_SetBackgroundColor( GLCD_COLOR_WHITE);
	GLCD_SetForegroundColor( GLCD_COLOR_LIGHT_GREY );
	GLCD_DrawChar( ( 9 + ulLedNum ) * 16, 6 * 24, 0x80 + 0 );     /* Circle empty */
	xSemaphoreGive( xLCDMutex );
#endif
}

/*----------------------------------------------------------------------------
  display message on GLCD
 *----------------------------------------------------------------------------*/
void vLCDDisplay( void *pvParameters )
{
	for (;;)
	{
		configASSERT( xSemaphoreTake( xLCDMutex, portMAX_DELAY ) == pdTRUE );
#ifdef __USE_LCD
		GLCD_SetBackgroundColor( GLCD_COLOR_BLUE );
		GLCD_SetForegroundColor( GLCD_COLOR_WHITE );
		GLCD_DrawString( 0 * 16, 0 * 24, "   V2M-MPS2 Demo    " );
		GLCD_DrawString( 0 * 16, 1 * 24, "      Blinky        " );
		GLCD_DrawString( 0 * 16, 2 * 24, "   DesignStart      " );
#endif
		xSemaphoreGive( xLCDMutex );
		vTaskDelay( MS_TO_TICKS( 4000 ) );

		configASSERT( xSemaphoreTake( xLCDMutex, portMAX_DELAY ) == pdTRUE );
#ifdef __USE_LCD
		GLCD_SetBackgroundColor( GLCD_COLOR_BLUE );
		GLCD_SetForegroundColor( GLCD_COLOR_RED );
		GLCD_DrawString( 0 * 16, 0 * 24, "   V2M-MPS2 Demo    " );
		GLCD_DrawString( 0 * 16, 1 * 24, "      Blinky        " );
		GLCD_DrawString( 0 * 16, 2 * 24, "   DesignStart      " );
#endif
		xSemaphoreGive( xLCDMutex );
		vTaskDelay( MS_TO_TICKS( 4000 ) );
  }
}

/*----------------------------------------------------------------------------
  blink LED
 *----------------------------------------------------------------------------*/
void vLEDBlinky( void *pvParameters )
{
	uint32_t ulLedMax    = LED_GetCount();
	uint32_t ulLedNum    = 0;

	for (;;)
	{
		configASSERT( xSemaphoreTake( xButtonLedSemaphore, portMAX_DELAY  ) == pdTRUE );
		LED_On( ulLedNum );                   /* Turn specified LED on */
		prvLedOnLcd( ulLedNum );

		configASSERT( xSemaphoreTake( xButtonLedSemaphore, portMAX_DELAY  ) == pdTRUE );
		LED_Off( ulLedNum );                  /* Turn specified LED off */
		prvLedOffLcd( ulLedNum );

		ulLedNum++;                           /* Change LED number */
		if ( ulLedNum >= ulLedMax )
		{
			ulLedNum = 0;                     /* Restart with first LED */
		}
	}
}

/*----------------------------------------------------------------------------
  check button state
 *----------------------------------------------------------------------------*/
void vGetButState( void *pvParameters )
{
	uint32_t ulButtonState;
	uint32_t ulButtonNum;
	uint32_t ulButtonMax;
	uint32_t ulButtonMsk;

	ulButtonMax = Buttons_GetCount();
	ulButtonMsk = ( 1U << ulButtonMax ) - 1;

	for(;;)
	{
		vTaskDelay( MS_TO_TICKS( 500 ) );

		do {
			ulButtonState = Buttons_GetState();
			for( ulButtonNum = 0; ulButtonNum < ulButtonMax; ulButtonNum++ )
			{
				if ( ulButtonState & ( 1ul << ulButtonNum ) )
				{
					prvButOnLcd( ulButtonNum );
				}
				else
				{
					prvButOffLcd( ulButtonNum );
				}
			}
		} while( ulButtonState & ulButtonMsk );

		xSemaphoreGive( xButtonLedSemaphore );
	}
}

int main (void)
{
	uint32_t button_msk = ( 1U << Buttons_GetCount() ) - 1;
#ifdef __USE_LCD
	uint32_t ledMax = LED_GetCount() - 1;
	uint32_t keyMax = Buttons_GetCount();
	int32_t i;
#endif

	SystemCoreClockUpdate();

	xButtonLedSemaphore = xSemaphoreCreateBinary();
	configASSERT( xButtonLedSemaphore != NULL );

	LED_Initialize();                         /* initalize LEDs */
	Buttons_Initialize();                     /* initalize Buttons */

#ifdef __USE_LCD
	xLCDMutex = xSemaphoreCreateMutex();
	configASSERT( xLCDMutex != NULL );

	GLCD_Initialize();                        /* Initialize the GLCD */

	/* display initial screen */
	GLCD_SetFont( &GLCD_Font_16x24 );
	GLCD_SetBackgroundColor( GLCD_COLOR_WHITE );
	GLCD_ClearScreen();
	GLCD_SetBackgroundColor( GLCD_COLOR_BLUE );
	GLCD_SetForegroundColor( GLCD_COLOR_RED );
	GLCD_DrawString(0 * 16, 0 * 24, "   V2M-MPS2 Demo    ");
	GLCD_DrawString(0 * 16, 1 * 24, "      Blinky        ");
	GLCD_DrawString(0 * 16, 2 * 24, "   DesignStart      ");

	GLCD_SetBackgroundColor( GLCD_COLOR_WHITE );
	GLCD_SetForegroundColor( GLCD_COLOR_BLACK );
	switch ( ( SCB->CPUID >> 4 ) & 0xFFF )
	{
	case 0xC20:
		GLCD_DrawString( 0 * 16, 4 * 24, "   ARM Cortex-M0    ");
		break;
	case 0xC60:
		GLCD_DrawString( 0 * 16, 4 * 24, "   ARM Cortex-M0+   ");
		break;
	case 0xC21:
		GLCD_DrawString( 0 * 16, 4 * 24, "   ARM Cortex-M1    ");
		break;
	case 0xC23:
		GLCD_DrawString( 0 * 16, 4 * 24, "   ARM Cortex-M3    ");
		break;
	case 0xC24:
		GLCD_DrawString( 0 * 16, 4 * 24, "   ARM Cortex-M4    ");
		break;
	case 0xC27:
		GLCD_DrawString( 0 * 16, 4 * 24, "   ARM Cortex-M7    ");
		break;
	default:
		GLCD_DrawString( 0 * 16, 4 * 24, "  unknown Cortex-M  ");
		break;
	}
	GLCD_DrawString( 2 * 16, 6 * 24, "LED:");
	GLCD_DrawString( 2 * 16, 7 * 24, "KEY:");

	GLCD_SetForegroundColor( GLCD_COLOR_LIGHT_GREY );
	for ( i = 0; i <= ledMax; i++ )
	{
		GLCD_DrawChar( ( 9 + i ) * 16, 6 * 24, 0x80 + 0);     /* draw empty circle */
	}
	for (i = 0; i <  keyMax; i++)
	{
		GLCD_DrawChar( ( 9 + i ) * 16, 7 * 24, 0x80 + 0);     /* draw empty circle */
	}
#endif

	/* Create a blinky tasks */
	xTaskCreate( vLCDDisplay,
		"LCDDisplayTask",
		1 * configMINIMAL_STACK_SIZE,
		NULL,
		tskIDLE_PRIORITY + 1,
		NULL );

	xTaskCreate( vGetButState,
		"GetButtonStateTask",
		1 * configMINIMAL_STACK_SIZE,
		NULL,
		tskIDLE_PRIORITY + 1,
		NULL );

	xTaskCreate( vLEDBlinky,
		"LEDBlinkTask",
		1 * configMINIMAL_STACK_SIZE,
		NULL,
		tskIDLE_PRIORITY + 1,
		NULL );

	/* Start the FreeRTOS scheduler */
	vTaskStartScheduler();
	for(;;);
}
